# -*- coding: utf-8 -*-
#
# AWL simulator - instructions
#
# Copyright 2012-2018 Michael Buesch <m@bues.ch>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

from __future__ import division, absolute_import, print_function, unicode_literals
#from awlsim.common.cython_support cimport * #@cy
from awlsim.common.compat import *

from awlsim.common.exceptions import *

from awlsim.core.instructions.main import * #+cimport
from awlsim.core.operatortypes import * #+cimport
from awlsim.core.operators import * #+cimport


class AwlInsn_PL(AwlInsn): #+cdef

	__slots__ = ()

	def __init__(self, cpu, rawInsn=None, **kwargs):
		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_PL, rawInsn, **kwargs)
		self.assertOpCount(1)
		if self.op0.operType != AwlOperatorTypes.IMM:
			raise AwlSimError("Immediate expected")

	def run(self): #+cdef
#@cy		cdef AwlOperator oper

		oper = self.op0
		if oper.width == 16:
			self.cpu.accu1.setWord(
				self.cpu.accu1.getSignedWord() +
				AwlMemoryObject_asScalar16(self.cpu.fetch(oper, self._widths_16)))
		elif oper.width == 32:
			self.cpu.accu1.setDWord(
				self.cpu.accu1.getSignedDWord() +
				AwlMemoryObject_asScalar32(self.cpu.fetch(oper, self._widths_32)))
		else:
			raise AwlSimError("Unexpected operator width")
